/*
    Copyright (c) 2002, 2003, bDistributed.com, Inc.
    All rights reserved.

    Redistribution and use in source and binary forms, with or
    without modification, are permitted provided that the following
    conditions are met:

    *   Redistributions of source code must retain the above
        copyright notice, this list of conditions and the following
        disclaimer.
    
    *   Redistributions in binary form must reproduce the above
        copyright notice, this list of conditions and the following
        disclaimer in the documentation and/or other materials
        provided with the distribution.
    
    *   Neither the name of bDistributed.com, Inc. nor the names of
        its contributors may be used to endorse or promote products
        derived from this software without specific prior written
        permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
    CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
    INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
    MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
    OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
    PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
    OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
    TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
    OF SUCH DAMAGE.
*/

#import <BDRuleEngine/BDRuleEngine.h>

#import "BDRuleEditorDocument.h"
#import "BDRuleEditorRule.h"

@implementation BDRuleEditorDocument

- (id)init
{
    if (self = [super init])
    {
        BDRuleEditorRule *initialRule;
        
        self.ruleSet = [NSMutableArray arrayWithCapacity:1];

        initialRule = [[BDRuleEditorRule alloc] init];
        
        [self.ruleSet addObject:initialRule];
    }
    
    return self;
}

- (void)dealloc
{
    self.ruleSet = nil;
}

- (NSString *)windowNibName
{
    return @"BDRuleEditorDocument";
}

- (NSData *)dataRepresentationOfType:(NSString *)aType
{
    NSData *data = nil;
    
    if ([aType isEqualToString:@"RuleSet"])
    {
        NSMutableDictionary *plist;
        BDRuleEditorRule *oneEditorRule;
        BDRuleModel *model;

        model = [[BDRuleModel alloc] init];
        
        for (oneEditorRule in self.ruleSet)
        {
            BDRule *rule;

            rule = [oneEditorRule rule];

            [model addToRules:rule];
        }
        
        plist = [NSMutableDictionary dictionaryWithCapacity:2];

        [model encodeIntoPropertyList:plist];
        
        data = (NSData *)CFBridgingRelease(CFPropertyListCreateXMLData(kCFAllocatorDefault,
                                                                       (__bridge CFPropertyListRef)plist));
        
        [self updateChangeCount:NSChangeCleared];
    }
    
    return data;
}

- (BOOL)loadDataRepresentation:(NSData *)data
                        ofType:(NSString *)aType
{
    BOOL loaded = NO;
    
    if ([aType isEqualToString:@"RuleSet"])
    {
        id plistRep;
        CFStringRef *errorString;
        
        plistRep = (id)CFBridgingRelease(CFPropertyListCreateFromXMLData(kCFAllocatorDefault,
                                                                         (__bridge CFDataRef)data,
                                                                         kCFPropertyListImmutable,
                                                                         (CFStringRef *)&errorString));

        if(plistRep)
        {
            loaded = [self loadFromPropertyList:plistRep];
        }
        else
        {
            loaded = NO;
        }
    }
    
    return loaded;
}


- (BOOL)loadFromPropertyList:(NSDictionary *)plist
{
    BOOL loaded = NO;
    BDRuleModel *model = nil;
    NSEnumerator *rulesEnumerator;
    BDRule *oneRule;

    model = [[BDRuleModel alloc] initWithPropertyList:plist
                                                 owner:self];

    [model awakeWithPropertyList:plist];

    // Empty the current rule set

    [self.ruleSet removeAllObjects];

    // Go through all of the rules in the rule set and deserialize them

    rulesEnumerator = [[model rules] objectEnumerator];

    while (oneRule = [rulesEnumerator nextObject])
    {
        BDRuleEditorRule *editorRule;

        editorRule = [[BDRuleEditorRule alloc] initWithRule:oneRule];

        [self.ruleSet addObject:editorRule];
    }

    loaded = YES;

    return loaded;
}

@end
