/*
    Copyright (c) 2002, 2003 bDistributed.com, Inc.
    All rights reserved.

    Redistribution and use in source and binary forms, with or
    without modification, are permitted provided that the following
    conditions are met:

    *   Redistributions of source code must retain the above
        copyright notice, this list of conditions and the following
        disclaimer.
    
    *   Redistributions in binary form must reproduce the above
        copyright notice, this list of conditions and the following
        disclaimer in the documentation and/or other materials
        provided with the distribution.
    
    *   Neither the name of bDistributed.com, Inc. nor the names of
        its contributors may be used to endorse or promote products
        derived from this software without specific prior written
        permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
    CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
    INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
    MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
    OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
    PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
    OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
    TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
    OF SUCH DAMAGE.
*/

#import <BDRuleEngine/BDRuleEngine.h>

#import "BDRuleEditorRule.h"

#import "TestBDRuleEditorRule.h"


@implementation TestBDRuleEditorRule

- (void)testInitWithRule
{
    BDRule *rule;
    BDRuleEditorRule *editorRule;
    
    rule = [BDRule ruleWithLhs:[NSPredicate predicateWithFormat:@"foo == 'bar'"]
                           rhs:[BDKeyAssignment assignmentWithKeyPath:@"baz.quux"
                                                                value:@"frobnitz"]
                      priority:73];

    editorRule = [[BDRuleEditorRule alloc] initWithRule:rule];

    XCTAssertEqualObjects(editorRule.lhs, @"foo == \"bar\"", @"");
    XCTAssertEqualObjects(editorRule.rhsKeyPath, @"baz.quux", @"");
    XCTAssertEqualObjects(editorRule.rhsValue, @"frobnitz", @"");
    XCTAssertEqualObjects(editorRule.assignmentClass, @"BDKeyAssignment", @"");
    XCTAssertTrue(editorRule.priority == 73, @"");
}

- (void)testGenerateRule
{
    BDRuleEditorRule *editorRule;
    BDRule *rule;

    editorRule = [[BDRuleEditorRule alloc] init];

    editorRule.lhs = @"a = b";
    editorRule.rhsKeyPath = @"c.d.e";
    editorRule.rhsValue = @"(e, f, g)";
    editorRule.assignmentClass = @"BDPropertyListAssignment";
    editorRule.priority = 12;

    rule = editorRule.rule;

    XCTAssertEqualObjects(rule.lhs.description, @"a == b", @"");
    XCTAssertEqualObjects(rule.rhs.keyPath, @"c.d.e", @"");
    XCTAssertEqualObjects(NSStringFromClass(rule.rhs.class), @"BDPropertyListAssignment", @"");
    XCTAssertEqualObjects(rule.rhs.value, @"(e, f, g)", @"");
    XCTAssertTrue(rule.priority == 12, @"");
}

@end
